==> chasen_2.0.orig.tar.gz: chasen-2.0.orig/mkchadic/mkary.c <==
/*
suffix array ץ

Υץθϡ
NLPRS '95  Invited Lecture  Kenneth W. Church
ΤΤѤޤ
*/
#include <fcntl.h>
#include <stdio.h>
#include <stdlib.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <sys/mman.h>

#define FNLEN 1000 /* ե̾Ĺ */
#define MODE_ON 1
#define MODE_OFF 0

void open_array_file();
size_t open_text_file();
void sort_array_file();
long make_first_array_file();
void usage();

/*=============
   ѿ
=============*/
char *text; /* оݤȤʤƥ */
long *suf; /* pointer(index)  */

int option_byline = MODE_OFF; /* ʸ˥ǥå */
char delimitter[20]; /* ԡñζڤ국 */
char *progname; /* program name */
int quiet_mode = MODE_OFF; /* åϤʤŤ⡼ɡ */
int comment_out_mode = MODE_OFF; /* #ǻϤޤԤϥȥ */
int no_sort_mode = MODE_OFF; /* Ȥʤ⡼ */
int sort_only_mode = MODE_OFF; /* Ȥʤ⡼ */
int bit_8_mode = MODE_ON; /* 2ХȰʸ⡼ */
int dict_mode = MODE_OFF;

FILE *ofd = NULL; /* ե */



int suffix_compare(a, b)
    long *a, *b;
{
  return strcmp(text + *a, text + *b);
/*
  char *s1 = text + *a, *s2 = text + *b;
  while(*s1 == *s2){
    s1++; s2++;
  }
  return (unsigned char)*s1-(unsigned char)*s2;
*/
}


main(argc, argv)
    int argc;
    char **argv;
{
  int fd = -1; /* ϥƥȥե */
  char in_fname[FNLEN]; /* ϥե̾ */
  char ary_fname[FNLEN]; /* ե̾ */
  size_t N;
  long i;
  long pointer_cnt;

  in_fname[0] = '\0';
  ary_fname[0] = '\0';

  progname = argv[0]; /* ץ̾ */

  /*================
    ץ
  ================*/
  if(argc <= 1){
    usage();
    exit(1);
  }
  while (argc > 1){
    if (argv[1][0] == '-')
      switch (argv[1][1]){
      case 'o': /* ϥե̾λ */
        if (argc == 2){ /* ʤȼդʤ */
          fprintf(stderr,"-o <filename> --- ϥե̾\n");
	  exit(1);
	}
	strcpy(ary_fname,argv[2]);
        argc--; argv++;
        break;
      case 'l': /* ˥ǥå */
        option_byline = MODE_ON;
        strcpy(delimitter,"\n");
        break;
      case 'w': /* ˥ǥå */
        option_byline = MODE_ON;
        strcpy(delimitter," \t\n\r\f{}.()~-`'");
        break;
      case 'c': /* ʸ˥ǥåʥǥեȡ */
        option_byline = MODE_OFF;
        break;
      case 'D': /*  */
	dict_mode = MODE_ON;
        break;
      case '8': /* 2ХȰʸԤʤʤ */
        bit_8_mode = MODE_OFF;
        break;
      case 'q': /* åϤʤ */
        quiet_mode = MODE_ON;
        break;
      case 'n': /* -ns Ȥʤ⡼ */
	if(argv[1][2] == 's') no_sort_mode = MODE_ON;
        break;
      case 's': /* -so Ȥʤ⡼ */
	if(argv[1][2] == 'o') sort_only_mode = MODE_ON;
        break;
      case '#': /* #ǻϤޤԤϥȥ */
        comment_out_mode = MODE_ON;
        break;
      default : /* 顼 */
        fprintf(stderr, "%c: ̵ʥץǤ\n", argv[1][1]);
	usage();
        exit(1);
      }
    else{
      strcpy(in_fname, argv[1]); /* ƥȥե̾ */
    }
    argc--; argv++;
  }


  /*** ƥȥե򳫤 (mmap) ***/
  N = open_text_file(in_fname);

  /*** arrayե򳫤 ***/
  if (ary_fname[0] == '\0') sprintf(ary_fname,"%s.ary",in_fname);
  if(sort_only_mode != MODE_ON) open_array_file(ary_fname);

  /*** ݥ(arrayե) ***/
  if(sort_only_mode == MODE_ON){
    /* arrayեϴ¸ߤΤǥȤ */
    if(quiet_mode == MODE_OFF) fprintf(stdout,"Array file exists...\n");
  } else {
    if(quiet_mode == MODE_OFF) fprintf(stdout,"Reading text file \"%s\"\n",in_fname);
    pointer_cnt = make_first_array_file((long)N);
    (void)fclose(ofd);
  }

  /*** arrayեΥݥ󥿤򥽡 ***/
  if(no_sort_mode == MODE_ON){ /* Ȥʤ⡼ */
    if(quiet_mode == MODE_OFF) fprintf(stdout,"No sort.\n");
  } else sort_array_file(ary_fname, pointer_cnt);

  if(quiet_mode == MODE_OFF) fprintf (stdout,"Done.\n");
  exit(0); /* ｪλ */
}


/******************************************************************************
  ƥȥե򳫤
 *****************************************************************************/
size_t open_text_file(fname)
    char *fname;
{
  struct stat stat_buf;
  int fd;
  size_t N;

  if ((fd = open(fname, O_RDONLY)) < 0){ /* ϥե */
    fprintf(stderr,"ե \"%s\" ץޤ\n", fname);
    exit(1);
  }

  (void)fstat(fd, &stat_buf);
  N = (size_t)stat_buf.st_size;

#ifdef NO_MMAP
  text = (caddr_t)malloc(N);
  read(fd, text, N);
#else
  if((text = mmap((caddr_t)0, N, PROT_READ, MAP_SHARED, fd, 0))
     == (caddr_t)-1){
    fprintf(stderr,"ERROR: text file mapping error.\n");
    exit(1);
  }
#endif
  return N;
}


/******************************************************************************
  arrayե򳫤
 *****************************************************************************/
void open_array_file(fname)
    char *fname;
{
  if((ofd = fopen(fname,"w")) == NULL){
    fprintf(stderr,"ե \"%s\" ץޤ\n", fname);
    exit(1);
  }
  if(quiet_mode == MODE_OFF) fprintf(stdout,"Save to \"%s\"\n",fname);
}


/******************************************************************************
 *   long make_first_array_file(long N);
 *
 * purpose
 *   ƥȥե򥹥󤷤ơݥ󥿤 arrayեǤФ
 *
 * parameters
 *   N : ƥȥեΥ
 *
 * return value
 *   arrayեΥ
 *
 * description
 *   ݥ󥿤򿶤ؿ(㤨СʸȤƬʤ)ϡ
 *   option_byline, delimitter ʤɤѿǷ롣
 *****************************************************************************/
long make_first_array_file(N)
    long N;
{
  long i, jj = 0;
  int last_char_is_delimitter = 1;
  int last_char_is_kanji = 0;
  int dic_ent = 1;

  /* printf("IN   N = %d\n",N);fflush(stdout);*/

  if (dict_mode == MODE_ON) {
      for(i = 0; i < N; i++){
	  /*printf("i  %d  %d\n",i,last_char_is_kanji);fflush(stdout);*/
	  if (dic_ent) {
	      if (!last_char_is_kanji && text[i] == '\t') dic_ent = 0;
	  } else {
	      if (!last_char_is_kanji && text[i] == '\n') {
		  dic_ent = 1;
		  continue;
	      }
	      else continue;
	  }
	  /* EUC2char */
	  if(bit_8_mode == MODE_ON
	     && last_char_is_kanji == 1
	     && (0x80 & text[i])) {
	      last_char_is_kanji = 0;
	  } else {
	      fwrite(&i, 1, sizeof(long), ofd);
	      jj++;
	      if((0x80 & text[i])) last_char_is_kanji = 1;
	  }
	  if(quiet_mode == MODE_ON || i == 0) continue;
	  if(!(i % 500000)) fputc('.', stdout);
	  if(!(i % 10000000)) printf(" %ldM\n",i/(1024*1024));
      }
      printf(" %ldM\n",i/(1024*1024));
  } else 
  if(option_byline == MODE_ON){ /* ԡ˥ǥå */
    for(i = 0; i < N; i++){
      if(strchr(delimitter, text[i]) != NULL && text[i] != '\0'){
	last_char_is_delimitter = 1;
      } else {
	if(last_char_is_delimitter == 1){ /* ʸڤʸʤ */
	  if(comment_out_mode && text[i] == '#'){
	    /* #ϤޤԤ̵  980319 */
	  } else {
	    fwrite(&i, 1, sizeof(long), ofd);
/*	    write(fd, &i, sizeof(long));*/
	    jj++;
	  }
	  last_char_is_delimitter = 0;
	}
      }
      if(quiet_mode == MODE_ON || i == 0) continue;
      if(!(i % 500000)) fputc('.', stdout);
      if(!(i % 10000000)) printf(" %ldM\n",i/(1024*1024));
    }
    printf(" %ldM\n",i/(1024*1024));
  } else {                    /* ʸ˥ǥå */
    for(i = 0; i < N; i++){
      /*printf("i  %d  %d\n",i,last_char_is_kanji);fflush(stdout);*/
      /* EUC2char */
      if(bit_8_mode == MODE_ON
	 && (0x80 & text[i]) != 0x00
	 && last_char_is_kanji == 1){
	last_char_is_kanji = 0;
      } else {
	fwrite(&i, 1, sizeof(long), ofd);
	jj++;
	if((0x80 & text[i]) != 0x00) last_char_is_kanji = 1;
      }
      if(quiet_mode == MODE_ON || i == 0) continue;
      if(!(i % 500000)) fputc('.', stdout);
      if(!(i % 10000000)) printf(" %ldM\n",i/(1024*1024));
    }
    printf(" %ldM\n",i/(1024*1024));
  }
  /* printf("OUT\n");fflush(stdout);*/

  return(jj);
}


/******************************************************************************
  ݥ󥿤򥽡
 *****************************************************************************/
void sort_array_file(ary_fname, pointer_cnt)
    char *ary_fname;
    long pointer_cnt;
{
  struct stat om_stat_buf;
  char *outmap;
  int omfd;
  size_t omsize;

  if ((omfd = open(ary_fname, O_RDWR)) < 0){  /* ϥե */
    fprintf(stderr,"ե \"%s\" ץޤ\n", ary_fname);
    exit(1);
  }

  (void)fstat(omfd, &om_stat_buf);
  omsize = (size_t)om_stat_buf.st_size;
  if(sort_only_mode == MODE_ON) pointer_cnt = omsize / sizeof(long);
  /* omsize = pointer_cnt * sizeof(int); ǤOK */
  /* printf("%ld %ld\n",pointer_cnt*sizeof(long),omsize);*/
#ifdef NO_MMAP
  outmap = (caddr_t)malloc(omsize);
  read(omfd, outmap, omsize);
#else
  if((outmap = mmap((caddr_t)0, omsize, PROT_READ | PROT_WRITE, MAP_SHARED, omfd, 0)) == (caddr_t)-1){
    fprintf(stderr,"ERROR: array file mapping error.\n");
    exit(1);
  }
#endif
  suf = (long *)outmap;

  if(quiet_mode == MODE_OFF) fprintf (stdout,"Sorting...\n");
  qsort(suf, (size_t)pointer_cnt, sizeof(long),
	(int (*)())suffix_compare);

  if(quiet_mode == MODE_OFF) fprintf (stdout,"Saving...\n");

  close(omfd);
}


/*
   usage --- Ȥ
*/
void usage(){
    static char *message[] = {
	"\n",
	"Version 1.4 980602 (SUFARY Version 2.0)\n\n",
	"USAGE   mkary [ -l [-#] ] [ -w ] [ -c ] [ -q ] [ -ns ] [ -so ] [ -8 ] [ -o filename ] filename\n",
	"OPTION\n",
	"  -o <filename> : ϥե ( default  stdout )\n",
	"  -l            : Ԥǥ ( \"\\n\" Ƕڤ )\n",
	"  -w            : ñǥ ( \" \",\"\\t\",\"\\n\" Ƕڤ )\n",
	"  -c            : ʸ()ǥ ( default )\n",
	"  -q            : åʤ\n",
	"  -ns           : Ȥʤ(No Sort)\n",
	"  -so           : Ȥ(Sort Only)\n",
	"  -8            : 2ХȰʸԤʤʤ\n",
	"  -#            : #ǻϤޤԤϥȥ(ԥȥ⡼ɤΤ)\n\n",
	NULL
    };
    char **mes;

    for (mes = message; *mes; mes++)
      fputs(*mes, stderr);
}

==> chasen-2.2.1.tar.gz: chasen-2.2.1/mkchadic/mkary.c <==
/*
suffix array ץ

Υץθϡ
NLPRS '95  Invited Lecture  Kenneth W. Church
ΤΤѤޤ
*/
#include <fcntl.h>
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <sys/param.h>
#ifdef HAVE_MMAP
#include <sys/mman.h>
#endif

#if defined _WIN32 && ! defined __CYGWIN__
typedef unsigned char *caddr_t;
#endif

#define FNLEN 1000 /* ե̾Ĺ */
#define MODE_ON 1
#define MODE_OFF 0

void open_array_file();
size_t open_text_file();
void sort_array_file();
long make_first_array_file();
void usage();

/*=============
   ѿ
=============*/
char *text; /* оݤȤʤƥ */
long *suf; /* pointer(index)  */

int option_byline = MODE_OFF; /* ʸ˥ǥå */
char delimitter[20]; /* ԡñζڤ국 */
char *progname; /* program name */
int quiet_mode = MODE_OFF; /* åϤʤŤ⡼ɡ */
int comment_out_mode = MODE_OFF; /* #ǻϤޤԤϥȥ */
int no_sort_mode = MODE_OFF; /* Ȥʤ⡼ */
int sort_only_mode = MODE_OFF; /* Ȥʤ⡼ */
int bit_8_mode = MODE_ON; /* 2ХȰʸ⡼ */
int dict_mode = MODE_OFF;

FILE *ofd = NULL; /* ե */



int suffix_compare(a, b)
    long *a, *b;
{
    long ha, hb;

    ha = ntohl(*a);
    hb = ntohl(*b);
    return strcmp(text + ha, text + hb);
/*
  char *s1 = text + *a, *s2 = text + *b;
  while(*s1 == *s2){
    s1++; s2++;
  }
  return (unsigned char)*s1-(unsigned char)*s2;
*/
}

int main(argc, argv)
    int argc;
    char **argv;
{
  char in_fname[FNLEN]; /* ϥե̾ */
  char ary_fname[FNLEN]; /* ե̾ */
  size_t N;
  long pointer_cnt;

  in_fname[0] = '\0';
  ary_fname[0] = '\0';

  progname = argv[0]; /* ץ̾ */

  /*================
    ץ
  ================*/
  if(argc <= 1){
    usage();
    exit(1);
  }
  while (argc > 1){
    if (argv[1][0] == '-')
      switch (argv[1][1]){
      case 'o': /* ϥե̾λ */
        if (argc == 2){ /* ʤȼդʤ */
          fprintf(stderr,"-o <filename> --- ϥե̾\n");
	  exit(1);
	}
	strcpy(ary_fname,argv[2]);
        argc--; argv++;
        break;
      case 'l': /* ˥ǥå */
        option_byline = MODE_ON;
        strcpy(delimitter,"\n");
        break;
      case 'w': /* ˥ǥå */
        option_byline = MODE_ON;
        strcpy(delimitter," \t\n\r\f{}.()~-`'");
        break;
      case 'c': /* ʸ˥ǥåʥǥեȡ */
        option_byline = MODE_OFF;
        break;
      case 'D': /*  */
	dict_mode = MODE_ON;
        break;
      case '8': /* 2ХȰʸԤʤʤ */
        bit_8_mode = MODE_OFF;
        break;
      case 'q': /* åϤʤ */
        quiet_mode = MODE_ON;
        break;
      case 'n': /* -ns Ȥʤ⡼ */
	if(argv[1][2] == 's') no_sort_mode = MODE_ON;
        break;
      case 's': /* -so Ȥʤ⡼ */
	if(argv[1][2] == 'o') sort_only_mode = MODE_ON;
        break;
      case '#': /* #ǻϤޤԤϥȥ */
        comment_out_mode = MODE_ON;
        break;
      default : /* 顼 */
        fprintf(stderr, "%c: ̵ʥץǤ\n", argv[1][1]);
	usage();
        exit(1);
      }
    else{
      strcpy(in_fname, argv[1]); /* ƥȥե̾ */
    }
    argc--; argv++;
  }


  /*** ƥȥե򳫤 (mmap) ***/
  N = open_text_file(in_fname);

  /*** arrayե򳫤 ***/
  if (ary_fname[0] == '\0') sprintf(ary_fname,"%s.ary",in_fname);
  if(sort_only_mode != MODE_ON) open_array_file(ary_fname);

  /*** ݥ(arrayե) ***/
  if(sort_only_mode == MODE_ON){
    /* arrayեϴ¸ߤΤǥȤ */
    if(quiet_mode == MODE_OFF) fprintf(stdout,"Array file exists...\n");
  } else {
    if(quiet_mode == MODE_OFF) fprintf(stdout,"Reading text file \"%s\"\n",in_fname);
    pointer_cnt = make_first_array_file((long)N);
    (void)fclose(ofd);
  }

  /*** arrayեΥݥ󥿤򥽡 ***/
  if(no_sort_mode == MODE_ON){ /* Ȥʤ⡼ */
    if(quiet_mode == MODE_OFF) fprintf(stdout,"No sort.\n");
  } else sort_array_file(ary_fname, pointer_cnt);

  if(quiet_mode == MODE_OFF) fprintf (stdout,"Done.\n");
  exit(0); /* ｪλ */
}


/******************************************************************************
  ƥȥե򳫤
 *****************************************************************************/
size_t open_text_file(fname)
    char *fname;
{
  struct stat stat_buf;
  int fd;
  size_t N;

  if ((fd = open(fname, O_RDONLY)) < 0){ /* ϥե */
    fprintf(stderr,"ե \"%s\" ץޤ\n", fname);
    exit(1);
  }

  (void)fstat(fd, &stat_buf);
  N = (size_t)stat_buf.st_size;

#ifdef HAVE_MMAP
  if((text = mmap((caddr_t)0, N, PROT_READ, MAP_SHARED, fd, 0))
     == (caddr_t)-1){
    fprintf(stderr,"ERROR: text file mapping error.\n");
    exit(1);
  }
#else
  text = (caddr_t)malloc(N);
  read(fd, text, N);
#endif
  return N;
}


/******************************************************************************
  arrayե򳫤
 *****************************************************************************/
void open_array_file(fname)
    char *fname;
{
  if((ofd = fopen(fname,"w")) == NULL){
    fprintf(stderr,"ե \"%s\" ץޤ\n", fname);
    exit(1);
  }
  if(quiet_mode == MODE_OFF) fprintf(stdout,"Save to \"%s\"\n",fname);
}


/******************************************************************************
 *   long make_first_array_file(long N);
 *
 * purpose
 *   ƥȥե򥹥󤷤ơݥ󥿤 arrayեǤФ
 *
 * parameters
 *   N : ƥȥեΥ
 *
 * return value
 *   arrayեΥ
 *
 * description
 *   ݥ󥿤򿶤ؿ(㤨СʸȤƬʤ)ϡ
 *   option_byline, delimitter ʤɤѿǷ롣
 *****************************************************************************/
long make_first_array_file(N)
    long N;
{
  long i, jj = 0, index;
  int last_char_is_delimitter = 1;
  int last_char_is_kanji = 0;
  int dic_ent = 1;

  /* printf("IN   N = %d\n",N);fflush(stdout);*/

  if (dict_mode == MODE_ON) {
      for(i = 0; i < N; i++){
	  /*printf("i  %d  %d\n",i,last_char_is_kanji);fflush(stdout);*/
	  if (dic_ent) {
	      if (!last_char_is_kanji && text[i] == '\t') dic_ent = 0;
	  } else {
	      if (!last_char_is_kanji && text[i] == '\n') {
		  dic_ent = 1;
		  continue;
	      }
	      else continue;
	  }
	  /* EUC2char */
	  if(bit_8_mode == MODE_ON
	     && last_char_is_kanji == 1
	     && (0x80 & text[i])) {
	      last_char_is_kanji = 0;
	  } else {
	      index = htonl(i);
	      fwrite(&index, 1, sizeof(long), ofd);
	      jj++;
	      if((0x80 & text[i])) last_char_is_kanji = 1;
	  }
	  if(quiet_mode == MODE_ON || i == 0) continue;
	  if(!(i % 500000)) fputc('.', stdout);
	  if(!(i % 10000000)) printf(" %ldM\n",i/(1024*1024));
      }
      printf(" %ldM\n",i/(1024*1024));
  } else 
  if(option_byline == MODE_ON){ /* ԡ˥ǥå */
    for(i = 0; i < N; i++){
      if(strchr(delimitter, text[i]) != NULL && text[i] != '\0'){
	last_char_is_delimitter = 1;
      } else {
	if(last_char_is_delimitter == 1){ /* ʸڤʸʤ */
	  if(comment_out_mode && text[i] == '#'){
	    /* #ϤޤԤ̵  980319 */
	  } else {
	    index = htonl(i);
	    fwrite(&index, 1, sizeof(long), ofd);
/*	    write(fd, &i, sizeof(long));*/
	    jj++;
	  }
	  last_char_is_delimitter = 0;
	}
      }
      if(quiet_mode == MODE_ON || i == 0) continue;
      if(!(i % 500000)) fputc('.', stdout);
      if(!(i % 10000000)) printf(" %ldM\n",i/(1024*1024));
    }
    printf(" %ldM\n",i/(1024*1024));
  } else {                    /* ʸ˥ǥå */
    for(i = 0; i < N; i++){
      /*printf("i  %d  %d\n",i,last_char_is_kanji);fflush(stdout);*/
      /* EUC2char */
      if(bit_8_mode == MODE_ON
	 && (0x80 & text[i]) != 0x00
	 && last_char_is_kanji == 1){
	last_char_is_kanji = 0;
      } else {
	index = htonl(i);
	fwrite(&index, 1, sizeof(long), ofd);
	jj++;
	if((0x80 & text[i]) != 0x00) last_char_is_kanji = 1;
      }
      if(quiet_mode == MODE_ON || i == 0) continue;
      if(!(i % 500000)) fputc('.', stdout);
      if(!(i % 10000000)) printf(" %ldM\n",i/(1024*1024));
    }
    printf(" %ldM\n",i/(1024*1024));
  }
  /* printf("OUT\n");fflush(stdout);*/

  return(jj);
}


/******************************************************************************
  ݥ󥿤򥽡
 *****************************************************************************/
void sort_array_file(ary_fname, pointer_cnt)
    char *ary_fname;
    long pointer_cnt;
{
  struct stat om_stat_buf;
  char *outmap;
  int omfd;
  size_t omsize;

  if ((omfd = open(ary_fname, O_RDWR)) < 0){  /* ϥե */
    fprintf(stderr,"ե \"%s\" ץޤ\n", ary_fname);
    exit(1);
  }

  (void)fstat(omfd, &om_stat_buf);
  omsize = (size_t)om_stat_buf.st_size;
  if(sort_only_mode == MODE_ON) pointer_cnt = omsize / sizeof(long);
  /* omsize = pointer_cnt * sizeof(int); ǤOK */
  /* printf("%ld %ld\n",pointer_cnt*sizeof(long),omsize);*/
#ifdef HAVE_MMAP
  if((outmap = mmap((caddr_t)0, omsize, PROT_READ | PROT_WRITE, MAP_SHARED, omfd, 0)) == (caddr_t)-1){
    fprintf(stderr,"ERROR: array file mapping error.\n");
    exit(1);
  }
#else
  outmap = (caddr_t)malloc(omsize);
  read(omfd, outmap, omsize);
#endif
  suf = (long *)outmap;

  if(quiet_mode == MODE_OFF) fprintf (stdout,"Sorting...\n");
  qsort(suf, (size_t)pointer_cnt, sizeof(long),
	(int (*)())suffix_compare);

  if(quiet_mode == MODE_OFF) fprintf (stdout,"Saving...\n");

  close(omfd);
}


/*
   usage --- Ȥ
*/
void usage(){
    static char *message[] = {
	"\n",
	"Version 1.4 980602 (SUFARY Version 2.0)\n\n",
	"USAGE   mkary [ -l [-#] ] [ -w ] [ -c ] [ -q ] [ -ns ] [ -so ] [ -8 ] [ -o filename ] filename\n",
	"OPTION\n",
	"  -o <filename> : ϥե ( default  stdout )\n",
	"  -l            : Ԥǥ ( \"\\n\" Ƕڤ )\n",
	"  -w            : ñǥ ( \" \",\"\\t\",\"\\n\" Ƕڤ )\n",
	"  -c            : ʸ()ǥ ( default )\n",
	"  -q            : åʤ\n",
	"  -ns           : Ȥʤ(No Sort)\n",
	"  -so           : Ȥ(Sort Only)\n",
	"  -8            : 2ХȰʸԤʤʤ\n",
	"  -#            : #ǻϤޤԤϥȥ(ԥȥ⡼ɤΤ)\n\n",
	NULL
    };
    char **mes;

    for (mes = message; *mes; mes++)
      fputs(*mes, stderr);
}
